/*
 * Copyright (c) 2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: NotificationHandler.java,v 1.5 2007/06/23 15:19:15 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.lead;

import java.net.URI;
import java.net.URISyntaxException;

import org.apache.xmlbeans.XmlObject;
import org.xmlpull.v1.builder.XmlElement;

import xsul.XmlConstants;
import xsul.invoker.DynamicInfosetInvokerException;
import xsul.lead.LeadContextHeader;
import xsul.message_router.MessageContext;
import xsul.xbeans_util.XBeansUtil;
import xsul.xhandler.BaseHandler;
import xsul5.MLogger;
import edu.indiana.extreme.lead.workflow_tracking.Notifier;
import edu.indiana.extreme.lead.workflow_tracking.NotifierFactory;
import edu.indiana.extreme.lead.workflow_tracking.common.ConstructorProps;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationContext;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationEntity;
import edu.indiana.extreme.lead.workflow_tracking.util.MessageUtil;
import edu.indiana.extreme.xbaya.XBayaConstants;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class NotificationHandler extends BaseHandler {

    private static final MLogger logger = MLogger.getLogger();

    private static final String INVOKING_MESSAGE = "Invoking a workflow";

    private static final String RECEIVE_RESULT_MESSAGE = "A workflow finished successfully.";

    private static final String RECEIVE_FAULT_MESSAGE = "A workflow failed.";

    private LeadContextHeader leadContext;

    private Notifier notifier;

    private InvocationContext invocationContext;

    /**
     * Constructs a NotificationHandler.
     * 
     * @param leadContext
     */
    public NotificationHandler(LeadContextHeader leadContext) {
        super(NotificationHandler.class.getName());
        this.leadContext = leadContext;
        ConstructorProps props = MessageUtil
                .createConstructorPropsFromLeadContext(leadContext);
        this.notifier = NotifierFactory.createNotifier(props);
    }

    /**
     * @see xsul.xhandler.BaseHandler#processOutgoingXml(org.xmlpull.v1.builder.XmlElement,
     *      xsul.message_router.MessageContext)
     */
    @Override
    public boolean processOutgoingXml(XmlElement soapEnvelope,
            MessageContext context) throws DynamicInfosetInvokerException {
        logger.finest("soapEnvelope: "
                + XMLUtil.xmlElementToString(soapEnvelope));

        URI myWorkflowID = null;
        URI myServiceID = URI.create(XBayaConstants.APPLICATION_SHORT_NAME);
        String userDN = this.leadContext.getUserDn();
        if (userDN != null || userDN.trim().length() == 0) {
            String serviceIDAsString = XBayaConstants.APPLICATION_SHORT_NAME
                    + ":" + userDN.trim();
            try {
                myServiceID = new URI(null, null, serviceIDAsString, null);
            } catch (URISyntaxException e) {
                logger.caught(e);
            }
        }
        String myNodeID = null;
        Integer myTimestep = null;
        InvocationEntity myEntity = this.notifier.createEntity(myWorkflowID,
                myServiceID, myNodeID, myTimestep);

        URI serviceWorkflowID = null;
        URI serviceServiceID = this.leadContext.getWorkflowId();
        if (serviceServiceID == null) {
            serviceServiceID = URI.create("NoWorkflowIDSet");
        }
        String serviceNodeID = this.leadContext.getNodeId();
        Integer serviceTimestep = null;
        String timeStep = this.leadContext.getTimeStep();
        if (timeStep != null) {
            try {
                serviceTimestep = new Integer(this.leadContext.getTimeStep());
            } catch (NumberFormatException e) {
                logger.caught(e);
            }
        }
        InvocationEntity serviceEntity = this.notifier.createEntity(
                serviceWorkflowID, serviceServiceID, serviceNodeID,
                serviceTimestep);

        XmlElement soapHeader = soapEnvelope.element(null,
                XmlConstants.S_HEADER);
        XmlElement soapBody = soapEnvelope.element(null, XmlConstants.S_BODY);
        XmlObject headerObject = null;
        if (soapHeader != null) {
            headerObject = XBeansUtil.xmlElementToXmlObject(soapHeader);
        }
        XmlObject bodyObject = XBeansUtil.xmlElementToXmlObject(soapBody);

        this.invocationContext = this.notifier.invokingService(myEntity,
                serviceEntity, headerObject, bodyObject, INVOKING_MESSAGE);
        return super.processOutgoingXml(soapEnvelope, context);
    }

    /**
     * @see xsul.xhandler.BaseHandler#processIncomingXml(org.xmlpull.v1.builder.XmlElement,
     *      xsul.message_router.MessageContext)
     */
    @Override
    public boolean processIncomingXml(XmlElement soapEnvelope,
            MessageContext context) throws DynamicInfosetInvokerException {
        logger.finest("soapEnvelope: "
                + XMLUtil.xmlElementToString(soapEnvelope));

        XmlElement soapHeader = soapEnvelope.element(null,
                XmlConstants.S_HEADER);
        XmlObject headerObject = null;
        if (soapHeader != null) {
            headerObject = XBeansUtil.xmlElementToXmlObject(soapHeader);
        }

        XmlElement soapBody = soapEnvelope.element(null, XmlConstants.S_BODY);
        XmlObject bodyObject = XBeansUtil.xmlElementToXmlObject(soapBody);
        XmlElement faultElement = soapBody.element(null, "Fault");
        if (faultElement == null) {
            this.notifier.receivedResult(this.invocationContext, headerObject,
                    bodyObject, RECEIVE_RESULT_MESSAGE);
        } else {
            this.notifier.receivedFault(this.invocationContext, headerObject,
                    bodyObject, RECEIVE_FAULT_MESSAGE);
        }

        return super.processIncomingXml(soapEnvelope, context);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2007 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
